require "lope2d"
candy = require "candy"

function love.load()
    gr = love.graphics
    kb = love.keyboard
    mo = love.mouse
    to = love.touch
    ma = love.math

    colors = {
        black = {0, 0, 0},
        darkGray = {48, 48, 48},
        white = {255, 255, 255},
        almostWhite = {248, 248, 248},
        red = {208, 16, 0},
        green = {0, 104, 0},
        lightGreen = {40, 192, 32},
    }

    win = {w = gr.getWidth(), h = gr.getHeight()}
    canvasW = 144
    canvasH = 256
    canvas = gr.newCanvas(canvasW, canvasH, "normal")
    sx = win.w / canvasW -- x scale
    sy = win.h / canvasH -- y scale

    wt = 60 -- wall thickness
    cr = 13 -- candy radius

    --ball = lope.newBody("dynamic", "circle", canvasW / 2, canvasH / 2, 10)
    candies = {}

    wallTop     = lope.newBody("static", "rectangle", canvasW / 2, -wt / 2, canvasW, wt)
    wallBottom  = lope.newBody("static", "rectangle", canvasW / 2, canvasH + wt / 2, canvasW, wt)
    wallLeft    = lope.newBody("static", "rectangle", -wt / 2, canvasH / 2, wt, canvasH + wt * 2)
    wallRight   = lope.newBody("static", "rectangle", canvasW + wt / 2, canvasH / 2, wt, canvasH + wt * 2)

    MOUSE_LEFT = 1
    MOUSE_RIGHT = 2
    MOUSE_MIDDLE = 3



    timerEnterCandy = 0
    delayEnterCandy = .2
end

function love.update(dt)
    lope.update(dt)

    timerEnterCandy = timerEnterCandy + dt

    if timerEnterCandy >= delayEnterCandy then
        timerEnterCandy = 0

        local newCandy = candy.new(
            ma.random(cr, canvasW - cr),
            ma.random(cr, 50),
            cr
        )

        newCandy.body = lope.newBody(
            "dynamic",
            "circle",
            newCandy.x,
            newCandy.y,
            newCandy.radius
        )

        table.insert(candies, newCandy)
    end

    -- update candies:
    for k, candy in ipairs(candies) do
        candy.x, candy.y = candy.body:getX(), candy.body:getY()
    end
end

function love.draw()
    gr.setColor(colors.white)
    gr.setCanvas(canvas)

    gr.setColor(colors.black)
    gr.rectangle("fill", 0, 0, canvasW, canvasH)
    --debug:
    gr.setColor(colors.white)
    --lope.draw(ball)

    -- draw candies:
    for k, candy in ipairs(candies) do
        gr.setColor(colors.red)
        if candy.selected then
            gr.setColor(colors.lightGreen)
        end
        gr.circle("line", candy.x, candy.y, candy.radius)

        if mo.isDown(MOUSE_LEFT) then
            local selectedCandy = getSelectedCandy()

            if selectedCandy then
                gr.line(
                    selectedCandy.x,
                    selectedCandy.y,
                    mo.getX() / sx,
                    mo.getY() / sy)
            end
        end
    end

    gr.setCanvas()
    gr.setColor(colors.white)
    gr.draw(canvas, 0, 0, 0, sx, sy)

    gr.print(fps())
end

function love.keypressed(k)
    if k == "escape" then
        quit()
    end
end

function love.mousepressed(x, y, btn)
    if btn == MOUSE_LEFT then
        for k, candy in ipairs(candies) do
            if candy:overIt(mo.getX() / sx, mo.getY() / sy) then
                candy.selected = true
            else
                candy.selected = false
            end
        end
    end
end

function love.mousereleased(x, y, btn)
    if btn == MOUSE_LEFT then
        --debug:
        --ball.body:applyLinearImpulse(ma.random(-500, 500), ma.random(-500, 500))

        for k, candy in ipairs(candies) do
            local selectedCandy = getSelectedCandy()

            if selectedCandy then
                selectedCandy.body:applyLinearImpulse(
                    -(selectedCandy.x - mo.getX() / sx) / 2,
                    -(selectedCandy.y - mo.getY() / sy) / 2
                )
            end
        end
    end
end








function getSelectedCandy()
    for k, candy in ipairs(candies) do
        if candy.selected then
            return candy
        end
    end
end

function fps()
    return love.timer.getFPS()
end

function quit()
    love.event.quit()
end
